library(readxl)
library(tidyverse)

# Load dataset
data_ <- read_xlsx("../data/S4 extra_clinical data.xlsx")
colnames(data_) <- c("Sample ID", "Gender", "Age", "Clinical Category",
                   "TMPRSS2 (Val197Met)", "Clinical known co-morbidities",
                   "comorbidity_coded")

#Build contingency table for extra mutations:
# (Y217Y, P100P, G422G, T112T, G8V, I293I, intronic)
cn   <- c("severe", "mild")
rn   <-  c("g1", "a.wt", "Y217Y", "a.wt", "P100P", "a.wt", "G422G",
           "a.wt", "T112T", "a.wt", "G8V", "a.wt", "I293I", "a.wt", "intronic")
bad  <- c(672, 16, 671, 17, 645, 43, 545, 143, 238, 426, 147, 540, 2, 646)
good <- c(469, 15, 466, 18, 455, 29, 378, 106, 165, 299, 114, 370, 0, 461)

preptab <- rbind(bad, good) %>%
  as_tibble %>%
  mutate(group1 = cn, .before = everything()) %>%
  set_names(rn) %>%
  pivot_longer(-g1, names_to = "g2", values_to = "count") %>%
  mutate(metag = rep(c(1:7, 1:7), each = 2)) %>%
  arrange(metag)
###

# Create G296G contingency table

cn <- c("wt", "Gly296Gly")
rn <- c("mild", "severe")
good <- c(306, 178)
bad  <- c(490, 198)

g296g <- rbind(good, bad)
rownames(g296g) <- rn
colnames(g296g) <- cn

# Set up the augmented dataset for analysis of v197m mutation
t_data <- data_ %>%
  mutate(`Age class` = case_when(
                          Gender == 0 ~ ifelse(Age < 61, "Young M", "Old M"),
                          Gender == 1 ~ ifelse(Age < 58, "Young F", "Old F")
                          ),
  Gender = factor(Gender, label = c("Male", "Female")),
  `Clinical Category` = factor(`Clinical Category`,
                          label = c("not hospitalised",
                                     "no Oxygen therapy",
                                     "Oxygen therapy",
                                     "CPAP/BiPAP",
                                     "Intubation"
                                     ))
  )

# helper for selecting genes, producing different contingency tables
# for sex/ages/genotype combination and performing statistical tests over those
contab_helper <- function(gene, mild, severe, meto) {
  gene_ <- rlang:::enquo(gene)
  dat_markfac <- t_data %>%
    mutate("General class" = case_when(`Clinical Category` %in% levels(`Clinical Category`)[mild] ~ "Mild",
                                       `Clinical Category` %in% levels(`Clinical Category`)[severe] ~ "Severe"),
           Genotype = factor(!!gene_, label = c("Wild Type", "Mutated"))
    ) %>%
  filter(!is.na(`General class`))


  prep_tabs1 <- dat_markfac %>%
    dplyr::select(Genotype, contains("class")) %>%
    group_by_all() %>%
    summarise(Freq = n(), .groups = "drop")

  tabs_1 <- xtabs(Freq ~ Genotype + `General class` + `Age class`, prep_tabs1)

  tabs_2 <- dat_markfac %>%
    dplyr::select(Genotype, `General class`, Gender) %>%
    group_by_all() %>%
    table

  tabs_3 <- dat_markfac %>%
    count(`General class`, `Genotype`, name = "Freq") %>%
    xtabs(Freq ~Genotype + `General class`, data = .)


  age    <- contab_extraction(tabs_1, meto)
  gender <- contab_extraction(tabs_2, meto)
  global <- epitools::oddsratio(tabs_3, method = meto, verbose = T)
  res <- list(age = age, gender = gender, global = global)
  return(res)
}

# helper for contingency table extraction from oddsratio object
contab_extraction <- function(..x, meto_) {
  list_ <- lapply(seq(dim(..x)[3]), function(x) ..x[, , x])

  names(list_) <- dimnames(..x)[3][[1]]

  vals  <- lapply(list_, epitools::oddsratio, method = meto_, verbose = T)

  avals <- lapply(list_, function(..x)
                    rbind(cbind(..x, marginal_rows = rowSums(..x)),
                          marginal_cols = c(colSums(..x), sum(..x))
                          )
                   )

  return(list(list_, vals, avals))
}

# Build contingency table and test for significance for G296G
g296g <- epitools::oddsratio(t(as.matrix(g296g)), method = "wald", verbose = T)

# Build contingency table and test for significance for various mutations
# and factor combinations
v197m        <- contab_helper(`TMPRSS2 (Val197Met)`, 1:2, 3:5, "wald")

extra_muts <- group_split(preptab, metag) %>%
  lapply(., function(x) xtabs(count ~ g2 + g1, data = x)) %>%
  lapply(., epitools::oddsratio, method = "wald", verbose = T)

#Main text tables
table_1a <- g296g
table_1b <- v197m$global

table_2a <- v197m$age[[2]]$`Young M`
table_2b <- v197m$age[[2]]$`Old F`

table_3a <- data_ %>%
  filter(Gender == 0,
         Age <= 60,
         comorbidity_coded == 1) %>%
  select(`TMPRSS2 (Val197Met)`, `Clinical Category`) %>%
  mutate(`Clinical Category` = ifelse(`Clinical Category` > 1, 2, 1)) %>%
  table %>%
  epitools::oddsratio(method = "wald")

table_3b <- data_ %>%
  filter(Gender == 1,
         Age >= 58,
         comorbidity_coded == 1) %>%
  select(`TMPRSS2 (Val197Met)`, `Clinical Category`) %>%
  mutate(`Clinical Category` = ifelse(`Clinical Category` > 1, 2, 1)) %>%
  table %>%
  epitools::oddsratio(method = "wald")

#SupMat tables

#S2 (extra_tabs_other_muts)
y217y    <- extra_muts[[1]][c("data", "measure", "p.value")]
p100p    <- extra_muts[[2]][c("data", "measure", "p.value")]
g422g    <- extra_muts[[3]][c("data", "measure", "p.value")]
t112t    <- extra_muts[[4]][c("data", "measure", "p.value")]
g8v      <- extra_muts[[5]][c("data", "measure", "p.value")]
i293i    <- extra_muts[[6]][c("data", "measure", "p.value")]
intronic <- extra_muts[[7]][c("data", "measure", "p.value")]

#S3 (extra_tabs_non_significant)
ns1_oldm <- v197m$age[[2]]$`Old M`
ns2_youf <- v197m$age[[2]]$`Young F`
