library(readxl)
library(tidyverse)
library(patchwork)

# Load dataset
data_ <- read_xlsx("../data/S4 extra_clinical data.xlsx")
colnames(data_) <- c("Sample ID", "Gender", "Age", "Clinical Category",
                   "TMPRSS2 (Val197Met)", "Clinical known co-morbidities",
                   "comorbidity_coded")

# Set up the augmented dataset for analysis of v197m mutation
t_data <- data_ %>%
  mutate(`Clinical Category` = factor(`Clinical Category`,
                          label = c("not hospitalised",
                                     "no Oxygen therapy",
                                     "Oxygen therapy",
                                     "CPAP/BiPAP",
                                     "Intubation"
                                     )),
         "General class" = case_when(`Clinical Category` %in% levels(`Clinical Category`)[1:2] ~ "Mild",
                                     `Clinical Category` %in% levels(`Clinical Category`)[3:5] ~ "Severe"),
         Genotype = factor(`TMPRSS2 (Val197Met)`, label = c("Wild Type", "Mutated")),
         Gender = factor(Gender, label = c("Male", "Female"))
    ) %>%
  filter(!is.na(`General class`))


# Function for age-based split and table generation
split_tabgen <- function(x, y) {
  x %>%
    mutate(`Age class` = ifelse(Age < y, "Young", "Old")) %>%
    dplyr::select(Genotype, contains("class")) %>%
    group_by_all() %>%
    summarise(Freq = n(), .groups = "drop") %>%
    xtabs(Freq ~ Genotype + `General class` +
                      `Age class`, .)
}

# Function for the extraction of OR (with ci) and p value
extractor <- function(x) c(x$measure[-1, ], p = x$p.value[2, 3])

# Function for the age sweep
perform_sweep <- function(z) {
  #Build the sweep object
  meta <- seq(40, 80, by = 1) %>%
    enframe(value = "Age") %>%
    mutate(tables = map(Age, ~split_tabgen(z, .x))) %>%
    slice(-c(1, nrow(.))) %>%
    mutate(t_list = map(tables, ~list(old = .x[, , 1], young = .x[, , 2]))) %>%
    unnest_longer(t_list) %>%
    mutate(oras = map(t_list, ~{
                                epitools::oddsratio(.x, method = "wald")
                               }
                     )
          ) %>%
    mutate(thing = map(oras, extractor)) %>%
    select(name, Age, `Age class` = t_list_id, thing) %>%
    unnest_wider(thing)

  # Build the estimate plots
  estimplo <- ggplot(meta) +
   aes(x = Age, y = estimate, ymin = lower, ymax = upper) +
   geom_pointrange(aes(col = `Age class`)) +
   facet_wrap(~`Age class`)

  # Build the pvalue plots
  pplo    <- ggplot(meta) +
   aes(x = Age, y = p) +
   geom_point(aes(col = `Age class`))

  list(estimates = estimplo, pvals = pplo)
}

# Application to females
t_dat_fem <- filter(t_data, Gender == "Female")
fem <- perform_sweep(t_dat_fem)

# Application to males
t_dat_mal <- filter(t_data, Gender == "Male")
mal <- perform_sweep(t_dat_mal)

a <- fem$pvals +
  xlim(50, 69) +
  facet_wrap(~p < 0.05, scale = "free", nrow = 2) +
  theme_minimal() +
  ggtitle("females")

b <- mal$pvals +
  xlim(50, 69) +
  facet_wrap(~p < 0.05, scale = "free", nrow = 2) +
  theme_minimal() +
  ggtitle("males")

rawpa <- (a | b) +
  plot_layout(guides = "collect")

ggsave("raw_panel_wald.svg", rawpa, width = 14, height = 10, dpi = 300)

#the raw panel gets further elaborated in inkscape to produce Sup. Fig. S1

# Assemble and saves extended gender panels
fem_pan <- (fem$estimates / fem$pvals) +
  plot_annotation(title = "Females")
#o ggsave("fempan_years.png", fem_pan, width = 14, height = 7, dpi = 300)

mal_pan <- (mal$estimates / mal$pvals) +
  plot_annotation(title = "Males")
#o ggsave("malpan_years.png", mal_pan, width = 14, height = 7, dpi = 300)
