library(readxl)
library(tidyverse)

# Load dataset
data_ <- read_xlsx("../data/S4 extra_clinical data.xlsx")
colnames(data_) <- c("Sample ID", "Gender", "Age", "Clinical Category",
                   "TMPRSS2 (Val197Met)", "Clinical known co-morbidities",
                   "comorbidity_coded")

# Set up the augmented dataset for analysis of v197m mutation
t_data <- data_ %>%
  group_by(Gender) %>%
  mutate(`Age class` = case_when(Age <= quantile(Age)[2] ~ "Young (Q1)",
                                 Age  <= quantile(Age)[3] ~ "Adult(Q2)",
                                 Age  < quantile(Age)[4] ~ "Mature (Q3)",
                                 Age  >= quantile(Age)[4] ~ "Old(Q4)",
                                 TRUE ~"Outside")) %>%
  ungroup() %>%
  filter(`Age class` != "Outside") %>%
  mutate(`Clinical Category` = factor(`Clinical Category`,
                          label = c("not hospitalised",
                                     "no Oxygen therapy",
                                     "Oxygen therapy",
                                     "CPAP/BiPAP",
                                     "Intubation"
                                     )),
         "General class" = case_when(`Clinical Category` %in% levels(`Clinical Category`)[1:2] ~ "Mild",
                                     `Clinical Category` %in% levels(`Clinical Category`)[3:5] ~ "Severe"),
         Genotype = factor(`TMPRSS2 (Val197Met)`, label = c("Wild Type", "Mutated")),
         Gender = factor(Gender, label = c("Male", "Female"))
    ) %>%
  filter(comorbidity_coded == 1)

# Preparatory step for table production
prep_tabs1 <- t_data %>%
  dplyr::select(Genotype, contains("class"), Gender) %>%
  group_by_all() %>%
  summarise(Freq = n(), .groups = "drop")

# Make the tables
tabs <- xtabs(Freq ~ Genotype + `General class` +
                    `Age class` + `Gender`, prep_tabs1)

# Contingency tables for quartiles/sex.
# (SupMat S3 extra_tab_non_significant, except for Table 4a and 4b, see below)
tabs_list <- list(
                  q1_m = tabs[, , 1, 1], #Male, Adult
                  q2_m = tabs[, , 2, 1], #Male, Mature
                  q3_m = tabs[, , 3, 1], #Male, Old
                  q4_m = tabs[, , 4, 1], #Male, Young
                  q1_f = tabs[, , 1, 2], #Female, Adult
                  q2_f = tabs[, , 2, 2], #Female, Mature
                  q3_f = tabs[, , 3, 2], #Female, Old
                  q4_f = tabs[, , 4, 2]  #Female, Young
                 )
                   
# Calculate oddsratio for all tables
oddsratios <- lapply(tabs_list, epitools::oddsratio, method = "wald")

# Display tables, oddsratio with CI, and pvalues_infos.
contingency_tabs <- lapply(oddsratios, "[", 1)
oddsratio_w_ci   <- lapply(oddsratios, "[", 2)
pvalues_infos    <- lapply(oddsratios, "[", 3)

# Table 4a, young (Age <= 50) males
ym_contingency_tabs <- contingency_tabs[4]
ym_oddsratio_w_ci   <- oddsratio_w_ci[4]
ym_pvalues_infos    <- pvalues_infos[4]

# Table 4b, old (Age >= 71) females
of_contingency_tabs <- contingency_tabs[7]
of_oddsratio_w_ci   <- oddsratio_w_ci[7]
of_pvalues_infos    <- pvalues_infos[7]
